@@ -20,6 +20,7 @@ enum SupportedModules {
     Elf,
     Pe,
     Dotnet,
+    Crx,
 }
 
 #[derive(Debug, Clone, ValueEnum)]
@@ -111,6 +112,9 @@ pub fn exec_dump(args: &ArgMatches) -> anyhow::Result<()> {
         if !requested_modules.contains(&&SupportedModules::Pe) {
             module_output.pe = MessageField::none()
         }
+        if !requested_modules.contains(&&SupportedModules::Crx) {
+            module_output.crx = MessageField::none()
+        }
     } else {
         // Module was not specified, only show those that produced meaningful
         // results, the rest are cleared out.
@@ -131,6 +135,9 @@ pub fn exec_dump(args: &ArgMatches) -> anyhow::Result<()> {
         if !module_output.pe.is_pe() {
             module_output.pe = MessageField::none()
         }
+        if !module_output.crx.is_crx() {
+            module_output.crx = MessageField::none()
+        }
     }
 
     match output_format {
