@@ -121,6 +121,12 @@ pub fn scan() -> Command {
                 .help("Don't use memory-mapped files")
                 .long_help(help::NO_MMAP_LONG_HELP)
         )
+        .arg(
+            arg!(--"max-matches-per-pattern" <MATCHES>)
+                .help("Maximum number of matches per pattern")
+                .long_help(help::MAX_MATCHES_PER_PATTERN_LONG_HELP)
+                .value_parser(value_parser!(usize))
+        )
         .arg(
             arg!(-o --"output-format" <FORMAT>)
                 .help("Output format for results")
@@ -256,6 +262,8 @@ pub fn exec_scan(args: &ArgMatches, config: &Config) -> anyhow::Result<()> {
     let scan_list = args.get_flag("scan-list");
     let recursive = args.get_one::<usize>("recursive");
     let no_mmap = args.get_flag("no-mmap");
+    let max_matches_per_pattern =
+        args.get_one::<usize>("max-matches-per-pattern");
 
     let timeout =
         args.get_one::<u64>("timeout").map(|t| Duration::from_secs(*t));
@@ -359,7 +367,7 @@ pub fn exec_scan(args: &ArgMatches, config: &Config) -> anyhow::Result<()> {
                 as Box<dyn OutputHandler>
         }
         Some(OutputFormats::Ndjson) => {
-            Box::new(NdJsonOutputHandler::new(args.into()))
+            Box::new(NdjsonOutputHandler::new(args.into()))
         }
         None | Some(OutputFormats::Text) => {
             Box::new(TextOutputHandler::new(args.into()))
@@ -396,6 +404,10 @@ pub fn exec_scan(args: &ArgMatches, config: &Config) -> anyhow::Result<()> {
                 scanner.use_mmap(false);
             }
 
+            if let Some(max_matches_per_pattern) = max_matches_per_pattern {
+                scanner.max_matches_per_pattern(*max_matches_per_pattern);
+            }
+
             scanner
         },
         // File handler. Called for every file found while walking the path.
@@ -975,17 +987,17 @@ mod output_handler {
         }
     }
 
-    pub(super) struct NdJsonOutputHandler {
+    pub(super) struct NdjsonOutputHandler {
         output_options: OutputOptions,
     }
 
-    impl NdJsonOutputHandler {
+    impl NdjsonOutputHandler {
         pub(super) fn new(output_options: OutputOptions) -> Self {
             Self { output_options }
         }
     }
 
-    impl OutputHandler for NdJsonOutputHandler {
+    impl OutputHandler for NdjsonOutputHandler {
         fn on_file_scanned(
             &self,
             file_path: &Path,
