@@ -147,7 +147,7 @@ fn print_meta() {
         .arg("src/tests/testdata/dummy.file")
         .assert()
         .success()
-        .stdout("foo [string=\"foo\",bool=true,int=1,float=3.14] src/tests/testdata/dummy.file\n");
+        .stdout("foo [string=\"foo\",bool=true,int=1,float=3.14,regexp=\"foo\"] src/tests/testdata/dummy.file\n");
 }
 
 #[test]
@@ -245,7 +245,6 @@ fn ignore_module() {
   |
 1 | import "unknown"
   | ---------------- module `unknown` used here
-  |
 warning[unsupported_module]: module `unknown` is not supported
  --> src/tests/testdata/unknown_module.yar:5:6
   |
