@@ -412,11 +412,6 @@ pub struct EmptyPatternSet {
 #[associated_enum(CompileError)]
 #[error(code = "E017", title = "`entrypoint` is unsupported")]
 #[label("the `entrypoint` keyword is not supported anymore", error_loc)]
-#[label(
-    "use `pe.entry_point` or `elf.entry_point` or `macho.entry_point`",
-    error_loc,
-    Level::HELP
-)]
 pub struct EntrypointUnsupported {
     report: Report,
     error_loc: CodeLoc,
@@ -901,12 +896,14 @@ pub struct IncludeNotAllowed {
 ///
 /// # Example
 ///
+/// ```text
 /// error[E045]: arbitrary regular expression prefix  
 ///  --> line:3:11  
 ///   |  
 /// 3 |     $a = /.*foo/s  
 ///   |           ^^ this prefix can be arbitrarily long and matches all bytes  
-///   |  
+///   |
+/// ```  
 ///
 /// Regular expressions with such prefixes are problematic because YARA will
 /// report a match at every file offset from the start of the file up to where
