@@ -8,6 +8,7 @@ use std::rc::Rc;
 
 use bstr::{BString, ByteSlice};
 use itertools::Itertools;
+
 use yara_x_parser::ast;
 use yara_x_parser::ast::WithSpan;
 use yara_x_parser::Span;
@@ -26,7 +27,7 @@ use crate::compiler::ir::{
     Error, Expr, ExprId, Iterable, LiteralPattern, MatchAnchor, Pattern,
     PatternFlags, PatternIdx, PatternInRule, Quantifier, Range, RegexpPattern,
 };
-use crate::compiler::report::ReportBuilder;
+use crate::compiler::report::{Level, ReportBuilder};
 use crate::compiler::{
     warnings, CompileContext, CompileError, ForVars, TextPatternAsHex,
 };
@@ -444,10 +445,16 @@ fn expr_from_ast(
 ) -> Result<ExprId, CompileError> {
     let expr = match expr {
         ast::Expr::Entrypoint { span } => {
-            return Err(EntrypointUnsupported::build(
+            let mut err = EntrypointUnsupported::build(
                 ctx.report_builder,
                 ctx.report_builder.span_to_code_loc(span.clone()),
-            ));
+            );
+
+            err.report()
+                .new_section(Level::HELP, "use `pe.entry_point`, elf.entry_point` or `macho.entry_point`")
+                .patch(span.clone(), "pe.entry_point");
+
+            return Err(err);
         }
         ast::Expr::Filesize { .. } => ctx.ir.filesize(),
 
@@ -534,9 +541,7 @@ fn expr_from_ast(
                         ctx.ir.not(lhs),
                         format!(
                             "not {}",
-                            ctx.report_builder.get_snippet(
-                                &ctx.report_builder.span_to_code_loc(lhs_span)
-                            )
+                            ctx.report_builder.get_snippet(lhs_span)
                         ),
                     )),
                     (
@@ -546,29 +551,17 @@ fn expr_from_ast(
                         ctx.ir.not(rhs),
                         format!(
                             "not {}",
-                            ctx.report_builder.get_snippet(
-                                &ctx.report_builder.span_to_code_loc(rhs_span)
-                            )
+                            ctx.report_builder.get_snippet(rhs_span)
                         ),
                     )),
                     (
                         TypeValue::Bool { .. },
                         TypeValue::Integer { value: Const(1), .. },
-                    ) => Some((
-                        lhs,
-                        ctx.report_builder.get_snippet(
-                            &ctx.report_builder.span_to_code_loc(lhs_span),
-                        ),
-                    )),
+                    ) => Some((lhs, ctx.report_builder.get_snippet(lhs_span))),
                     (
                         TypeValue::Integer { value: Const(1), .. },
                         TypeValue::Bool { .. },
-                    ) => Some((
-                        rhs,
-                        ctx.report_builder.get_snippet(
-                            &ctx.report_builder.span_to_code_loc(rhs_span),
-                        ),
-                    )),
+                    ) => Some((rhs, ctx.report_builder.get_snippet(rhs_span))),
                     _ => None,
                 };
 
@@ -940,7 +933,6 @@ fn bool_expr_from_ast(
 ) -> Result<ExprId, CompileError> {
     ctx.one_shot_symbol_table = None;
 
-    let code_loc = ctx.report_builder.span_to_code_loc(ast.span());
     let expr = expr_from_ast(ctx, ast)?;
 
     match ctx.ir.get(expr).type_value() {
@@ -953,7 +945,7 @@ fn bool_expr_from_ast(
                     let style = ctx.report_builder.green_style();
                     format!(
                         "you probably meant {style}{}(){style:#}",
-                        ctx.report_builder.get_snippet(&code_loc)
+                        ctx.report_builder.get_snippet(ast.span())
                     )
                 });
 
@@ -961,7 +953,7 @@ fn bool_expr_from_ast(
                 ctx.report_builder,
                 "`bool`".to_string(),
                 "a function".to_string(),
-                code_loc,
+                ctx.report_builder.span_to_code_loc(ast.span()),
                 help,
             ));
         }
@@ -970,7 +962,7 @@ fn bool_expr_from_ast(
                 ctx.report_builder,
                 "`bool`".to_string(),
                 "a map".to_string(),
-                code_loc,
+                ctx.report_builder.span_to_code_loc(ast.span()),
                 None,
             ));
         }
@@ -979,7 +971,7 @@ fn bool_expr_from_ast(
                 ctx.report_builder,
                 "`bool`".to_string(),
                 "a struct".to_string(),
-                code_loc,
+                ctx.report_builder.span_to_code_loc(ast.span()),
                 None,
             ));
         }
@@ -988,7 +980,7 @@ fn bool_expr_from_ast(
                 ctx.report_builder,
                 "`bool`".to_string(),
                 "an array".to_string(),
-                code_loc,
+                ctx.report_builder.span_to_code_loc(ast.span()),
                 None,
             ));
         }
@@ -997,7 +989,7 @@ fn bool_expr_from_ast(
                 ctx.report_builder,
                 "`bool`".to_string(),
                 "a regexp".to_string(),
-                code_loc,
+                ctx.report_builder.span_to_code_loc(ast.span()),
                 None,
             ));
         }
@@ -1064,10 +1056,29 @@ fn of_expr_from_ast(
         }
     };
 
-    // If the quantifier expression is greater than the number of items,
-    // the `of` expression is always false.
     if let Quantifier::Expr(expr) = &quantifier {
         if let Some(value) = ctx.ir.get(*expr).try_as_const_integer() {
+            // The use of `0 of them` is not recommended, because is not clear
+            // if 0 or more patterns must match, or if none of them should
+            // match.
+            if value == 0 {
+                let mut warning = warnings::AmbiguousExpression::build(
+                    ctx.report_builder,
+                    ctx.report_builder.span_to_code_loc(of.span()),
+                );
+
+                warning
+                    .report()
+                    .new_section(
+                        Level::HELP,
+                        "consider using `none` instead of `0`",
+                    )
+                    .patch(of.quantifier.span(), "none");
+
+                ctx.warnings.add(|| warning)
+            }
+            // If the quantifier expression is greater than the number of items,
+            // the `of` expression is always false.
             if value > items.len() as i64 {
                 ctx.warnings.add(|| warnings::InvariantBooleanExpression::build(
                     ctx.report_builder,
@@ -1880,6 +1891,19 @@ fn re_error_to_compile_error(
                 ),
             )
         }
+        re::parser::Error::UnsupportedInUnicode { span } => {
+            InvalidRegexp::build(
+                report_builder,
+                err.to_string(),
+                report_builder.span_to_code_loc(
+                    regexp
+                        .span()
+                        .subspan(span.start.offset, span.end.offset)
+                        .offset(1),
+                ),
+                None,
+            )
+        }
         re::parser::Error::MixedGreediness {
             is_greedy_1,
             is_greedy_2,
