@@ -1,6 +1,6 @@
 use annotate_snippets::renderer::{AnsiColor, Color, DEFAULT_TERM_WIDTH};
 use annotate_snippets::{
-    renderer, Annotation, AnnotationKind, Group, Snippet,
+    renderer, Annotation, AnnotationKind, Group, Patch, Snippet,
 };
 use serde::ser::SerializeStruct;
 use serde::{Serialize, Serializer};
@@ -39,12 +39,6 @@ impl CodeLoc {
     pub(crate) fn new(source_id: Option<SourceId>, span: Span) -> Self {
         Self { source_id, span }
     }
-
-    /// Returns the span within the source code.
-    #[inline]
-    pub fn span(&self) -> &Span {
-        &self.span
-    }
 }
 
 /// Represents an error or warning report.
@@ -82,6 +76,14 @@ pub(crate) struct Report {
     title: String,
     labels: Vec<(Level, CodeLoc, String)>,
     footers: Vec<(Level, String)>,
+    sections: Vec<Section>,
+}
+
+#[derive(Clone)]
+pub(crate) struct Section {
+    level: Level,
+    title: String,
+    patches: Vec<(Span, String)>,
 }
 
 impl Report {
@@ -133,6 +135,32 @@ impl Report {
             .iter()
             .map(|(level, text)| Footer { level: level_as_text(level), text })
     }
+
+    pub(crate) fn new_section<T: Into<String>>(
+        &mut self,
+        level: Level,
+        title: T,
+    ) -> &mut Self {
+        self.sections.push(Section {
+            level,
+            title: title.into(),
+            patches: vec![],
+        });
+        self
+    }
+
+    pub(crate) fn patch<R: Into<String>>(
+        &mut self,
+        span: Span,
+        replacement: R,
+    ) -> &mut Self {
+        self.sections
+            .last_mut()
+            .unwrap()
+            .patches
+            .push((span, replacement.into()));
+        self
+    }
 }
 
 impl Serialize for Report {
@@ -243,7 +271,26 @@ impl Display for Report {
         };
 
         let renderer = renderer.term_width(self.max_width);
-        let text = renderer.render(&[group]);
+
+        let mut groups = vec![group];
+
+        for section in &self.sections {
+            let mut snippet = Snippet::source(src);
+
+            for (span, replacement) in &section.patches {
+                snippet = snippet.patch(Patch::new(span.range(), replacement))
+            }
+
+            groups.push(
+                section
+                    .level
+                    .clone()
+                    .secondary_title(&section.title)
+                    .element(snippet),
+            );
+        }
+
+        let text = renderer.render(&groups);
 
         write!(f, "{text}")
     }
@@ -428,14 +475,14 @@ impl ReportBuilder {
         source_id
     }
 
-    /// Returns the fragment of source code indicated by `code_loc`.
-    pub fn get_snippet(&self, code_loc: &CodeLoc) -> String {
-        let source_id = code_loc.source_id.expect("CodeLoc without source ID");
+    /// Returns the fragment from the current source code indicated by `span`.
+    pub fn get_snippet(&self, span: Span) -> String {
+        let source_id = self.get_current_source_id().unwrap();
         let code_cache = self.code_cache.read();
         let cache_entry = code_cache.get(&source_id).unwrap();
         let src = cache_entry.code.as_str();
 
-        src[code_loc.span().range()].to_string()
+        src[span.range()].to_string()
     }
 
     /// Creates a new error or warning report.
@@ -465,6 +512,7 @@ impl ReportBuilder {
             title,
             labels,
             footers,
+            sections: Vec::new(),
         }
     }
 }
@@ -476,7 +524,7 @@ fn level_as_text(level: &Level) -> &'static str {
         Level::INFO => "info",
         Level::NOTE => "note",
         Level::HELP => "help",
-        _ => panic!("unsupported level {:?}", level),
+        _ => panic!("unsupported level {level:?}"),
     }
 }
 
