@@ -1,11 +1,13 @@
-use crate::compiler::{IdentId, PatternId, RuleInfo};
-use crate::scanner::{ScanContext, ScannedData};
-use crate::{compiler, scanner, Rules};
-use bstr::{BStr, ByteSlice};
-use serde::{Deserialize, Serialize};
 use std::ops::Range;
 use std::slice::Iter;
 
+use bstr::{BStr, ByteSlice};
+use serde::{Deserialize, Serialize};
+
+use crate::compiler::{IdentId, PatternId, RuleInfo};
+use crate::scanner::{ScanContext, ScanState};
+use crate::{compiler, scanner, Rules};
+
 /// Kinds of patterns.
 #[derive(Serialize, Deserialize, Clone, Copy)]
 pub enum PatternKind {
@@ -19,8 +21,7 @@ pub enum PatternKind {
 
 /// A structure that describes a rule.
 pub struct Rule<'a, 'r> {
-    pub(crate) ctx: Option<&'a ScanContext<'r>>,
-    pub(crate) data: Option<&'a ScannedData<'a>>,
+    pub(crate) ctx: Option<&'a ScanContext<'r, 'a>>,
     pub(crate) rules: &'r Rules,
     pub(crate) rule_info: &'r RuleInfo,
 }
@@ -73,7 +74,6 @@ impl<'a, 'r> Rule<'a, 'r> {
         Patterns {
             ctx: self.ctx,
             rules: self.rules,
-            data: self.data,
             include_private: false,
             iterator: self.rule_info.patterns.iter(),
             len_non_private: self.rule_info.patterns.len()
@@ -248,8 +248,7 @@ impl<'r> Tag<'r> {
 /// [`Patterns::include_private`] if you want to include private patterns
 /// as well.
 pub struct Patterns<'a, 'r> {
-    ctx: Option<&'a ScanContext<'r>>,
-    data: Option<&'a ScannedData<'a>>,
+    ctx: Option<&'a ScanContext<'r, 'a>>,
     rules: &'r Rules,
     iterator: Iter<'a, (IdentId, PatternKind, PatternId, bool)>,
     /// True if the iterator should yield all patterns, including the
@@ -302,7 +301,6 @@ impl<'a, 'r> Iterator for Patterns<'a, 'r> {
                 return Some(Pattern {
                     ctx: self.ctx,
                     rules: self.rules,
-                    data: self.data,
                     ident_id: *ident_id,
                     pattern_id: *pattern_id,
                     kind: *pattern_kind,
@@ -315,8 +313,7 @@ impl<'a, 'r> Iterator for Patterns<'a, 'r> {
 
 /// Represents a pattern defined by a rule.
 pub struct Pattern<'a, 'r> {
-    ctx: Option<&'a ScanContext<'r>>,
-    data: Option<&'a ScannedData<'a>>,
+    ctx: Option<&'a ScanContext<'r, 'a>>,
     rules: &'r Rules,
     ident_id: IdentId,
     pattern_id: PatternId,
@@ -343,9 +340,9 @@ impl<'a, 'r> Pattern<'a, 'r> {
     }
 
     /// Returns the matches found for this pattern.
-    pub fn matches(&self) -> Matches<'a> {
+    pub fn matches(&self) -> Matches<'a, 'r> {
         Matches {
-            data: self.data,
+            ctx: self.ctx,
             iterator: self.ctx.and_then(|ctx| {
                 ctx.pattern_matches
                     .get(self.pattern_id)
@@ -356,33 +353,33 @@ impl<'a, 'r> Pattern<'a, 'r> {
 }
 
 /// Iterator that returns the matches for a pattern.
-pub struct Matches<'a> {
-    data: Option<&'a ScannedData<'a>>,
+pub struct Matches<'a, 'r> {
+    ctx: Option<&'a ScanContext<'r, 'a>>,
     iterator: Option<Iter<'a, scanner::Match>>,
 }
 
-impl<'a> Iterator for Matches<'a> {
-    type Item = Match<'a>;
+impl<'a, 'r> Iterator for Matches<'a, 'r> {
+    type Item = Match<'a, 'r>;
 
     fn next(&mut self) -> Option<Self::Item> {
         let iter = self.iterator.as_mut()?;
-        Some(Match { data: self.data?, inner: iter.next()? })
+        Some(Match { ctx: self.ctx?, inner: iter.next()? })
     }
 }
 
-impl ExactSizeIterator for Matches<'_> {
+impl ExactSizeIterator for Matches<'_, '_> {
     fn len(&self) -> usize {
         self.iterator.as_ref().map_or(0, |it| it.len())
     }
 }
 
 /// Represents a match.
-pub struct Match<'a> {
-    data: &'a ScannedData<'a>,
+pub struct Match<'a, 'r> {
+    ctx: &'a ScanContext<'r, 'a>,
     inner: &'a scanner::Match,
 }
 
-impl<'a> Match<'a> {
+impl<'a> Match<'a, '_> {
     /// Range within the original data where the match occurred.
     #[inline]
     pub fn range(&self) -> Range<usize> {
@@ -392,7 +389,12 @@ impl<'a> Match<'a> {
     /// Slice containing the data that matched.
     #[inline]
     pub fn data(&self) -> &'a [u8] {
-        self.data.as_ref().get(self.inner.range.clone()).unwrap()
+        let data = match &self.ctx.scan_state {
+            ScanState::Finished(snippets) => snippets.get(self.range()),
+            _ => None,
+        };
+
+        data.unwrap()
     }
 
     /// XOR key used for decrypting the data if the pattern had the `xor`
