@@ -9,13 +9,16 @@ use crate::modules::protos::dotnet::*;
 pub mod parser;
 
 #[module_main]
-fn main(data: &[u8], _meta: Option<&[u8]>) -> Dotnet {
+fn main(
+    data: &[u8],
+    _meta: Option<&[u8]>,
+) -> Result<Dotnet, ModuleError> {
     match parser::Dotnet::parse(data) {
-        Ok(dotnet) => dotnet.into(),
+        Ok(dotnet) => Ok(dotnet.into()),
         Err(_) => {
             let mut dotnet = Dotnet::new();
             dotnet.is_dotnet = Some(false);
-            dotnet
+            Ok(dotnet)
         }
     }
 }
