@@ -48,6 +48,19 @@ const _CS_MAGIC_DETACHED_SIGNATURE: u32 = 0xfade0cc1;
 const CS_MAGIC_BLOBWRAPPER: u32 = 0xfade0b01;
 const CS_MAGIC_EMBEDDED_ENTITLEMENTS: u32 = 0xfade7171;
 
+/// Mach-O symtol table flag constants
+pub const N_STAB: u8 = 0xe0; /* if any of these bits set, a symbolic debugging entry */
+const _N_PEXT: u8 = 0x10; /* private external symbol bit */
+pub const N_TYPE: u8 = 0x0e; /* mask for the type bits */
+pub const N_EXT: u8 = 0x01; /* external symbol bit, set for external symbols */
+
+/// Mach-o value flags for N_TYPE bits of the n_type field.
+const N_UNDF: u8 = 0x0; /* undefined, n_sect == NO_SECT */
+const N_ABS: u8 = 0x2; /* absolute, n_sect == NO_SECT */
+const N_SECT: u8 = 0xe; /* defined in section number n_sect */
+const _N_PBUD: u8 = 0xc; /* prebound undefined (defined in a dylib) */
+const N_INDR: u8 = 0xa; /* indirect */
+
 /// Mach-O export flag constants
 const EXPORT_SYMBOL_FLAGS_WEAK_DEFINITION: u64 = 0x00000004;
 const EXPORT_SYMBOL_FLAGS_REEXPORT: u64 = 0x00000008;
@@ -341,21 +354,27 @@ impl<'a> MachO<'a> {
             }
         }
 
-        if let Some(ref mut symtab) = macho.symtab {
+        if let Some(ref symtab) = macho.symtab {
             let str_offset = symtab.stroff as usize;
             let str_end = symtab.strsize as usize;
+            let sym_offset = symtab.symoff as usize;
+            let nsyms = symtab.nsyms;
 
             // We don't want the dyld_shared_cache ones for now
             if let Some(string_table) =
                 data.get(str_offset..str_offset.saturating_add(str_end))
             {
-                let strings: Vec<&'a [u8]> = string_table
-                    .split(|&c| c == b'\0')
-                    .map(|line| BStr::new(line).trim_end_with(|c| c == '\0'))
-                    .filter(|s| !s.trim().is_empty())
-                    .collect();
-
-                symtab.entries.extend(strings);
+                if let Some(symbol_table) = data.get(sym_offset..) {
+                    if let Err(_err) =
+                        macho.parse_symtab(string_table, symbol_table, nsyms)
+                    {
+                        #[cfg(feature = "logging")]
+                        error!("Error parsing Mach-O file: {:?}", _err);
+                        // fail silently if it fails, data was not formatted
+                        // correctly but parsing should still proceed for
+                        // everything else
+                    };
+                }
             }
         }
 
@@ -405,6 +424,10 @@ impl<'a> MachO<'a> {
             }
         }
 
+        // imports defined at `weak_offset` can be duplicative, meaning
+        // they can exist in `bind_offset` _and_ `weak_offset` or one of them.
+        // To avoid duplicates in the imports list, keep track of seen imports
+        let mut seen: HashSet<_> = HashSet::new();
         for (offset, size) in macho
             .dyld_info
             .as_ref()
@@ -421,7 +444,8 @@ impl<'a> MachO<'a> {
             if let Some(import_data) =
                 data.get(offset..offset.saturating_add(size))
             {
-                if let Err(_err) = macho.parse_imports(import_data) {
+                if let Err(_err) = macho.parse_imports(import_data, &mut seen)
+                {
                     #[cfg(feature = "logging")]
                     error!("Error parsing Mach-O file: {:?}", _err);
                     // fail silently if it fails, data was not formatted
@@ -824,6 +848,7 @@ impl<'a> MachOFile<'a> {
                 stroff,
                 strsize,
                 entries: Vec::new(),
+                nlists: Vec::new(),
             },
         )
     }
@@ -1089,6 +1114,63 @@ impl<'a> MachOFile<'a> {
         )
     }
 
+    /// Parser that parses the nlist structures that are defined by the offsets
+    /// in LC_SYMTAB.
+    fn nlist(
+        &self,
+    ) -> impl Parser<&'a [u8], Output = Nlist, Error = NomError<'a>> + '_ {
+        map(
+            (
+                u32(self.endianness),                   // n_strx
+                u8,                                     // n_type
+                u8,                                     // n_sect
+                u16(self.endianness),                   // n_desc
+                uint(self.endianness, self.is_32_bits), // n_value
+            ),
+            |(n_strx, n_type, n_sect, n_desc, n_value)| Nlist {
+                n_strx,
+                n_type,
+                n_sect,
+                n_desc,
+                n_value,
+            },
+        )
+    }
+
+    /// Parser that parses the symbol table which includes the nlist structure
+    /// and the relevant string from the string table as defined in LC_SYMTAB.
+    fn parse_symtab(
+        &mut self,
+        string_table: &'a [u8],
+        symbol_table: &'a [u8],
+        count: u32,
+    ) -> IResult<&'a [u8], ()> {
+        let mut data = symbol_table;
+        let mut n;
+
+        for _ in 0..count {
+            (data, n) = self.nlist().parse(data)?;
+            if let Some(symtab) = self.symtab.as_mut() {
+                if let Some(string_data) =
+                    string_table.get(n.n_strx as usize..)
+                {
+                    let (_, string_value) = map(
+                        (take_till(|b| b == b'\x00'), tag("\x00")),
+                        |(s, _)| s,
+                    )
+                    .parse(string_data)?;
+
+                    if !string_value.is_empty() {
+                        symtab.entries.push(string_value);
+                        symtab.nlists.push(n);
+                    }
+                }
+            }
+        }
+
+        Ok((data, ()))
+    }
+
     /// Parser that parses the exports at the offsets defined within
     /// LC_DYLD_INFO, LC_DYLD_INFO_ONLY, and LC_DYLD_EXPORTS_TRIE.
     fn parse_exports(&mut self, data: &'a [u8]) -> IResult<&'a [u8], ()> {
@@ -1171,7 +1253,11 @@ impl<'a> MachOFile<'a> {
     }
 
     /// Parser that parses the imports at the offsets defined within LC_DYLD_INFO and LC_DYLD_INFO_ONLY
-    fn parse_imports(&mut self, data: &'a [u8]) -> IResult<&'a [u8], ()> {
+    fn parse_imports(
+        &mut self,
+        data: &'a [u8],
+        seen: &mut HashSet<&'a str>,
+    ) -> IResult<&'a [u8], ()> {
         let mut remainder: &[u8] = data;
         let mut entry: u8;
 
@@ -1202,7 +1288,10 @@ impl<'a> MachOFile<'a> {
                     .parse(remainder)?;
                     remainder = import_remainder;
                     if let Ok(import) = strr.to_str() {
-                        self.imports.push(import.to_string());
+                        if !seen.contains(import) {
+                            self.imports.push(import.to_string());
+                            seen.insert(import);
+                        }
                     }
                 }
                 _ => {}
@@ -1648,6 +1737,15 @@ struct Symtab<'a> {
     stroff: u32,
     strsize: u32,
     entries: Vec<&'a [u8]>,
+    nlists: Vec<Nlist>,
+}
+
+struct Nlist {
+    n_strx: u32,
+    n_type: u8,
+    n_sect: u8,
+    n_desc: u16,
+    n_value: u64,
 }
 
 struct Dysymtab {
@@ -1820,14 +1918,14 @@ fn convert_to_version_string(decimal_number: u32) -> String {
     let major = decimal_number >> 16;
     let minor = (decimal_number >> 8) & 0xFF;
     let patch = decimal_number & 0xFF;
-    format!("{}.{}.{}", major, minor, patch)
+    format!("{major}.{minor}.{patch}")
 }
 
 /// Convert a decimal number representation to a build version string representation.
 fn convert_to_build_tool_version(decimal_number: u32) -> String {
     let a = decimal_number >> 16;
     let b = (decimal_number >> 8) & 0xff;
-    format!("{}.{}", a, b)
+    format!("{a}.{b}")
 }
 
 /// Convert a decimal number representation to a source version string
@@ -1839,12 +1937,12 @@ fn convert_to_source_version_string(decimal_number: u64) -> String {
     let c = (decimal_number >> 20) & mask;
     let d = (decimal_number >> 10) & mask;
     let e = decimal_number & mask;
-    format!("{}.{}.{}.{}.{}", a, b, c, d, e)
+    format!("{a}.{b}.{c}.{d}.{e}")
 }
 
 /// Parses CMS certificates from a BER-encoded blob that are embedded in the
 /// Mach-O binary.
-fn parse_certificates(ber_blob: &[u8]) -> BerResult<Vec<Certificate>> {
+fn parse_certificates(ber_blob: &[u8]) -> BerResult<'_, Vec<Certificate>> {
     parse_ber_sequence_defined_g(|ber_blob: &[u8], _| {
         let (remainder, _content_type) = parse_ber_oid(ber_blob)?;
 
@@ -1886,13 +1984,13 @@ fn parse_certificates(ber_blob: &[u8]) -> BerResult<Vec<Certificate>> {
 /// Parses a BER-encoded sequence of AlgorithmIdentifiers.
 fn parse_digest_algorithms(
     input: &[u8],
-) -> BerResult<Vec<AlgorithmIdentifier>> {
+) -> BerResult<'_, Vec<AlgorithmIdentifier<'_>>> {
     Ok(parse_ber_set_of_v(AlgorithmIdentifier::from_ber)(input)
         .map_err(|_| BerValueError)?)
 }
 
 /// Parses a BER-encoded sequence of ContentInfo objects.
-fn parse_content_info(input: &[u8]) -> BerResult {
+fn parse_content_info(input: &[u8]) -> BerResult<'_> {
     parse_ber_sequence(input)
 }
 
@@ -1941,11 +2039,11 @@ impl From<MachO<'_>> for protos::macho::Macho {
                 for (idx, c) in uuid.iter().enumerate() {
                     match idx {
                         3 | 5 | 7 | 9 => {
-                            uuid_str.push_str(format!("{:02X}", c).as_str());
+                            uuid_str.push_str(format!("{c:02X}").as_str());
                             uuid_str.push('-');
                         }
                         _ => {
-                            uuid_str.push_str(format!("{:02X}", c).as_str());
+                            uuid_str.push_str(format!("{c:02X}").as_str());
                         }
                     }
                 }
@@ -1969,6 +2067,60 @@ impl From<MachO<'_>> for protos::macho::Macho {
             result.entitlements.extend(m.entitlements.clone());
             result.exports.extend(m.exports.clone());
             result.imports.extend(m.imports.clone());
+
+            // If the exports are empty, iterate the symbol table entries to
+            // check like dyld_info does:
+            // https://github.com/apple-oss-distributions/dyld/blob/main/other-tools/dyld_info.cpp#L560-L617
+            if m.dyld_export_trie.is_none() && m.dyld_info.is_none() {
+                if let Some(symtab) = &m.symtab {
+                    result.exports.extend(
+                        symtab
+                            .nlists
+                            .iter()
+                            .enumerate()
+                            .filter_map(|(idx, nlist)| {
+                                let t = nlist.n_type & N_TYPE;
+                                if (nlist.n_type & N_EXT != 0)
+                                    && ((t == N_SECT)
+                                        || (t == N_ABS)
+                                        || (t == N_INDR))
+                                    && ((nlist.n_type & N_STAB) == 0)
+                                {
+                                    symtab.entries.get(idx)
+                                } else {
+                                    None
+                                }
+                            })
+                            .map(|sym| BStr::new(sym).to_string()),
+                    )
+                }
+            }
+
+            // If the imports are empty, iterate the symbol table entries to
+            // check for undefined symbols like dyld_info does:
+            // https://github.com/apple-oss-distributions/dyld/blob/main/other-tools/dyld_info.cpp#L372-L398
+            if m.dyld_chain_fixups.is_none() && m.dyld_info.is_none() {
+                if let Some(symtab) = &m.symtab {
+                    result.imports.extend(
+                        symtab
+                            .nlists
+                            .iter()
+                            .enumerate()
+                            .filter_map(|(idx, nlist)| {
+                                let t = nlist.n_type & N_TYPE;
+                                if (t == N_UNDF)
+                                    && (nlist.n_type & N_STAB == 0)
+                                {
+                                    symtab.entries.get(idx)
+                                } else {
+                                    None
+                                }
+                            })
+                            .map(|sym| BStr::new(sym).to_string()),
+                    );
+                }
+            }
+
             result
                 .certificates
                 .extend(m.certificates.iter().map(|cert| cert.into()));
@@ -2027,11 +2179,11 @@ impl From<&MachOFile<'_>> for protos::macho::File {
             for (idx, c) in uuid.iter().enumerate() {
                 match idx {
                     3 | 5 | 7 | 9 => {
-                        uuid_str.push_str(format!("{:02X}", c).as_str());
+                        uuid_str.push_str(format!("{c:02X}").as_str());
                         uuid_str.push('-');
                     }
                     _ => {
-                        uuid_str.push_str(format!("{:02X}", c).as_str());
+                        uuid_str.push_str(format!("{c:02X}").as_str());
                     }
                 }
             }
@@ -2053,6 +2205,57 @@ impl From<&MachOFile<'_>> for protos::macho::File {
         result.entitlements.extend(macho.entitlements.clone());
         result.exports.extend(macho.exports.clone());
         result.imports.extend(macho.imports.clone());
+
+        // If the exports are empty, iterate the symbol table entries to check
+        // like dyld_info does:
+        // https://github.com/apple-oss-distributions/dyld/blob/main/other-tools/dyld_info.cpp#L560-L617
+        if macho.dyld_export_trie.is_none() && macho.dyld_info.is_none() {
+            if let Some(symtab) = &macho.symtab {
+                result.exports.extend(
+                    symtab
+                        .nlists
+                        .iter()
+                        .enumerate()
+                        .filter_map(|(idx, nlist)| {
+                            let t = nlist.n_type & N_TYPE;
+                            if (nlist.n_type & N_EXT != 0)
+                                && ((t == N_SECT)
+                                    || (t == N_ABS)
+                                    || (t == N_INDR))
+                                && ((nlist.n_type & N_STAB) == 0)
+                            {
+                                symtab.entries.get(idx)
+                            } else {
+                                None
+                            }
+                        })
+                        .map(|sym| BStr::new(sym).to_string()),
+                )
+            }
+        }
+
+        // If the imports are empty, iterate the symbol table entries to
+        // check for undefined symbols like dyld_info does:
+        // https://github.com/apple-oss-distributions/dyld/blob/main/other-tools/dyld_info.cpp#L372-L398
+        if macho.dyld_chain_fixups.is_none() && macho.dyld_info.is_none() {
+            if let Some(symtab) = &macho.symtab {
+                result.imports.extend(
+                    symtab
+                        .nlists
+                        .iter()
+                        .enumerate()
+                        .filter_map(|(idx, nlist)| {
+                            let t = nlist.n_type & N_TYPE;
+                            if (t == N_UNDF) && (nlist.n_type & N_STAB == 0) {
+                                symtab.entries.get(idx)
+                            } else {
+                                None
+                            }
+                        })
+                        .map(|sym| BStr::new(sym).to_string()),
+                );
+            }
+        }
         result
             .certificates
             .extend(macho.certificates.iter().map(|cert| cert.into()));
@@ -2139,9 +2342,23 @@ impl From<&Symtab<'_>> for protos::macho::Symtab {
         result.set_nsyms(symtab.nsyms);
         result.set_stroff(symtab.stroff);
         result.set_strsize(symtab.strsize);
+        // populate the entries
         result
             .entries
             .extend(symtab.entries.iter().map(|entry| entry.to_vec()));
+        result.nlists.extend(symtab.nlists.iter().map(|n| n.into()));
+        result
+    }
+}
+
+impl From<&Nlist> for protos::macho::Nlist {
+    fn from(nlist: &Nlist) -> Self {
+        let mut result = protos::macho::Nlist::new();
+        result.set_n_strx(nlist.n_strx);
+        result.set_n_type(nlist.n_type as u32);
+        result.set_n_sect(nlist.n_sect as u32);
+        result.set_n_desc(nlist.n_desc as u32);
+        result.set_n_value(nlist.n_value);
         result
     }
 }
