@@ -19,7 +19,6 @@ use crate::modules::protos::yara::exts::{
 };
 use crate::symbols::{Symbol, SymbolLookup};
 use crate::types::{Array, Map, StringConstraint, TypeValue};
-use crate::wasm::WasmExport;
 
 /// Each of the entries in an Access Control List (ACL)
 ///
@@ -115,6 +114,9 @@ pub(crate) struct Struct {
     /// True if this is the root structure. The root structure is the top-level
     /// structure that contains global variables and modules.
     is_root: bool,
+    /// The name of the protobuf type this enum was crated from. If the enum
+    /// was not created from a protobuf type, this is `None`.
+    protobuf_type_name: Option<String>,
 }
 
 impl SymbolLookup for Struct {
@@ -133,7 +135,11 @@ impl SymbolLookup for Struct {
 impl Struct {
     /// Creates a new, empty struct.
     pub fn new() -> Self {
-        Self { fields: IndexMap::new(), is_root: false }
+        Self {
+            fields: IndexMap::new(),
+            is_root: false,
+            protobuf_type_name: None,
+        }
     }
 
     /// Makes this the root structure.
@@ -142,6 +148,11 @@ impl Struct {
         self
     }
 
+    /// Returns the protobuf type this enum was created from, if any.
+    pub fn protobuf_type_name(&self) -> Option<&str> {
+        self.protobuf_type_name.as_deref()
+    }
+
     /// Adds a new field to the structure.
     ///
     /// The field name may be a dot-separated sequence of field names, like
@@ -206,8 +217,8 @@ impl Struct {
         }
     }
 
-    /// Adds a new fields to the structure corresponding to the values
-    /// in the given enum.
+    /// Adds new fields to the structure corresponding to the values in the
+    /// given enum.
     fn add_enum_fields(&mut self, enum_descriptor: &EnumDescriptor) {
         let mut enclosing_msg = enum_descriptor.enclosing_message();
         let mut path = Vec::new();
@@ -277,10 +288,28 @@ impl Struct {
     pub fn field_entry_by_name(
         &mut self,
         name: String,
-    ) -> indexmap::map::Entry<String, StructField> {
+    ) -> indexmap::map::Entry<'_, String, StructField> {
         self.fields.entry(name)
     }
 
+    pub fn enum_substructures<F>(&mut self, f: &mut F)
+    where
+        F: FnMut(&mut Struct),
+    {
+        f(self);
+        for field in self.fields.values_mut() {
+            match &mut field.type_value {
+                TypeValue::Struct(s) => {
+                    Rc::<Struct>::get_mut(s).unwrap().enum_substructures(f);
+                }
+                TypeValue::Array(a) => {
+                    Rc::<Array>::get_mut(a).unwrap().enum_substructures(f);
+                }
+                _ => {}
+            }
+        }
+    }
+
     /// Creates a [`Struct`] from a protobuf message descriptor.
     ///
     /// Receives the [`MessageDescriptor`] corresponding to the protobuf
@@ -329,7 +358,7 @@ impl Struct {
         msg_descriptor: &MessageDescriptor,
         msg: Option<&dyn MessageDyn>,
         generate_fields_for_enums: bool,
-    ) -> Self {
+    ) -> Rc<Self> {
         let syntax = msg_descriptor.file_descriptor().syntax();
         let mut fields = Vec::new();
 
@@ -405,26 +434,11 @@ impl Struct {
             }
         }
 
-        let mut new_struct = Self { fields: field_index, is_root: false };
-
-        // Get the methods implemented for this struct type.
-        let methods = WasmExport::get_methods(msg_descriptor.full_name());
-
-        // For each method implemented by this struct field, add a field
-        // to the struct of function type. Methods cannot have the same name
-        // as an existing field.
-        for (name, method) in methods {
-            if new_struct
-                .add_field(name, TypeValue::Func(Rc::new(method)))
-                .is_some()
-            {
-                panic!(
-                    "method `{}` has the same name than a field in `{}`",
-                    name,
-                    msg_descriptor.name()
-                )
-            };
-        }
+        let mut new_struct = Self {
+            fields: field_index,
+            is_root: false,
+            protobuf_type_name: Some(msg_descriptor.full_name().to_string()),
+        };
 
         if generate_fields_for_enums && Self::is_module_root(msg_descriptor) {
             let mut enums = IndexSet::new();
@@ -445,7 +459,7 @@ impl Struct {
             }
         }
 
-        new_struct
+        Rc::new(new_struct)
     }
 
     /// Returns true if the given message is the YARA module's root message.
@@ -823,7 +837,7 @@ impl Struct {
                         enum_as_fields,
                     )
                 };
-                TypeValue::Struct(Rc::new(structure))
+                TypeValue::Struct(structure)
             }
         }
     }
@@ -957,22 +971,22 @@ impl Struct {
                         repeated
                             .into_iter()
                             .map(|value| {
-                                Rc::new(Self::from_proto_descriptor_and_value(
+                                Self::from_proto_descriptor_and_value(
                                     msg_descriptor,
                                     value,
                                     enum_as_fields,
-                                ))
+                                )
                             })
                             .collect(),
                     )
                 } else {
-                    Array::Structs(vec![Rc::new(
+                    Array::Structs(vec![
                         Struct::from_proto_descriptor_and_msg(
                             msg_descriptor,
                             None,
                             enum_as_fields,
                         ),
-                    )])
+                    ])
                 }
             }
         };
@@ -1004,7 +1018,7 @@ impl Struct {
                 syntax,
             ),
             ty => {
-                panic!("maps in YARA can't have keys of type `{}`", ty);
+                panic!("maps in YARA can't have keys of type `{ty}`");
             }
         };
 
@@ -1081,7 +1095,7 @@ impl Struct {
         msg_descriptor: &MessageDescriptor,
         value: ReflectValueRef,
         enum_as_fields: bool,
-    ) -> Struct {
+    ) -> Rc<Self> {
         if let ReflectValueRef::Message(m) = value {
             Struct::from_proto_descriptor_and_msg(
                 msg_descriptor,
@@ -1119,7 +1133,7 @@ impl Struct {
         }
     }
 
-    fn value_as_string(value: ReflectValueRef) -> &[u8] {
+    fn value_as_string(value: ReflectValueRef<'_>) -> &[u8] {
         match value {
             ReflectValueRef::String(v) => v.as_bytes(),
             ReflectValueRef::Bytes(v) => v,
@@ -1155,10 +1169,9 @@ impl PartialEq for Struct {
 
 #[cfg(test)]
 mod tests {
-    use std::rc::Rc;
-
     use super::Struct;
     use crate::types::{Array, Type, TypeValue};
+    use std::rc::Rc;
 
     #[test]
     fn test_struct() {
@@ -1176,6 +1189,42 @@ mod tests {
 
         root.add_field("foo.bar", TypeValue::var_integer_from(1));
     }
+    #[test]
+    fn test_proto_struct() {
+        use protobuf::MessageFull;
+
+        use crate::modules::protos::test_proto2::TestProto2;
+
+        let mut structure = Struct::from_proto_descriptor_and_msg(
+            &TestProto2::descriptor(),
+            None,
+            true,
+        );
+
+        let structure = Rc::<Struct>::get_mut(&mut structure).unwrap();
+
+        let mut names = Vec::new();
+
+        structure.enum_substructures(&mut |s| {
+            names.push(s.protobuf_type_name().map(|n| n.to_string()));
+            println!("{:?}\n\n", s)
+        });
+
+        // TODO
+        assert_eq!(
+            vec![
+                Some("test_proto2.TestProto2".to_string()),
+                Some("test_proto2.NestedProto2".to_string()),
+                Some("test_proto2.NestedProto2".to_string()),
+                None,
+                None,
+                None,
+                None,
+                None,
+            ],
+            names
+        );
+    }
 
     #[test]
     fn struct_eq() {
